(document:surround "/std/frame")
(document:insert "/std/functions")
(document:envelop with-translation _ "alterator-xkb")


(frame:on-leave (lambda() (and (xkb-is-dirty?) 'need-confirm)))
(frame:on-apply (lambda() (do-apply)))


(define (on-load-error reason) '())

(define (convert-to-pair x)
  (cons (string->symbol (woo-get-option x 'name))
 	(woo-get-option x 'description)))

(define (lang-selector)
  (define-operation get-lang)
  (or (get-lang (fluid-ref generic-session))
      (list "en")))

(define (get-list name)
    (woo-catch
     (lambda()
       (sort
        (map convert-to-pair (woo-list name 'lang))
        (lambda (x y) (string<? (cdr x) (cdr y)))))
     on-load-error))

;extract main data from woobus
(define *xkb-models* (get-list "/xkb/models"))
(define *xkb-layouts* (get-list "/xkb/layouts"))
(define *xkb-option-groups* (get-list "/xkb/option_groups"))
(define *xkb-switches* (get-list "/xkb/option_groups/grp"))
(define *xkb-current* (woo-catch (lambda() (woo-read-first "/xkb/current"))  on-load-error))


;then process current data
(define (current->list name)
  (let ((value (woo-get-option *xkb-current* name)))
    (or (and (not-empty-string? value)
             (map sure-symbol (string-cut value #\,)))
 	'())))


;all current options we access via cells to avoid use of set!
(define (default-variants)
  (list
    (cons (string->symbol "")
	  (_ "default"))))


(define *selected-variants* (default-variants))

(define *current-switch* (let ((lst (current->list 'options)))
                           (make-cell (list-index (lambda (x) (member (car x) lst)) ;x one of the options
                                             *xkb-switches*))))

(define *current-options*
  (let* ((current-switch (cell-ref *current-switch*))
         (current-options (current->list 'options)))
    (make-cell
     (if (number? current-switch)
         (delq (car (list-ref *xkb-switches* current-switch)) current-options)
         current-options))))

(define *current-model* (make-cell (woo-get-option *xkb-current* 'model #f)))


(define *current-layouts*  (make-cell (current->list 'layout)))
(define *avail-layouts* (make-cell (map car (filter (lambda(x) (not (member (car x) (cell-ref *current-layouts*))))
                                         *xkb-layouts*))))

;current variants
(define (make-variants lst len)
  (if (null? lst)
      (map sure-symbol (make-list len ""))
      lst))

(define *current-variants* (make-cell (make-variants (current->list 'variant)
                                                     (length (cell-ref *current-layouts*)))))

(define (get-layout-name x)
  (or (cond-cdr (assq x *xkb-layouts*))
      (error "use unknown layout in config" x)))

;additional tools for lists
(define (list-remove lst num)
  (append (list-head lst num) (list-tail lst (+ num 1))))

(define (list-transpose! lst num)
  (let* ((tail (drop lst num))
 	 (elem1 (car tail))
 	 (elem2 (cadr tail)))
	
     (set-car! tail elem2)
     (set-car! (cdr tail) elem1)))

;enable-disable feeature
(define (xkb-dirty-toggle state) (apply-button activity state))

(define (xkb-mark-as-dirty) (xkb-dirty-toggle (> (current-layouts count) 0)))
(define (xkb-clear-dirty) (xkb-dirty-toggle #f))
(define (xkb-is-dirty?) (apply-button activity))


(define (remove-layout)
  (let ((number (current-layouts current)))
    (and (>= number 0)
	 (let ((layout (list-ref (cell-ref *current-layouts*) number)))
	   ;reset current layouts and appropriate listbox
	   (current-layouts remove number)
	   (cell-set! *current-layouts* (list-remove (cell-ref *current-layouts*) number))
	   (cell-set! *current-variants* (list-remove (cell-ref *current-variants*) number))
 	   ;reset available layouts and appropriate listbox
	   (cell-set! *avail-layouts* (sort
				   (append (cell-ref *avail-layouts*) (list layout))
				   (lambda (x y)
				     (string<? (get-layout-name x)
					       (get-layout-name y)))))
	   (avail-layouts rows (map get-layout-name (cell-ref *avail-layouts*)))
	   (xkb-mark-as-dirty)))))

(define (add-layout)
  (let ((number (avail-layouts current)))
    (and (>= number 0)
	 (let ((layout (list-ref (cell-ref *avail-layouts*) number)))
	   ;reset avail layouts
	   (avail-layouts remove number)
	   (cell-set! *avail-layouts* (list-remove (cell-ref *avail-layouts*) number))
	   ;reset current layouts
	   (cell-set! *current-layouts* (append (cell-ref *current-layouts*) (list layout)))
	   (cell-set! *current-variants* (append (cell-ref *current-variants*) (list (string->symbol ""))))
	   (current-layouts append-row (get-layout-name layout))
	   (xkb-mark-as-dirty)))))

(define (up-layout)
   (let ((number (- (current-layouts current) 1)))
     (and (>= number 0)
 	 (begin
 	   (list-transpose! (cell-ref *current-layouts*) number)
 	   (list-transpose! (cell-ref *current-variants*) number)
 	   (current-layouts rows (map get-layout-name (cell-ref *current-layouts*))
                            current number)
 	   (xkb-mark-as-dirty)))))

(define (down-layout)
  (let ((number (current-layouts current)))
    (and (>= number 0)
	 (< number (- (length (cell-ref *current-layouts*)) 1))
	 (begin
	   (list-transpose! (cell-ref *current-layouts*) number)
	   (list-transpose! (cell-ref *current-variants*) number)
	   (current-layouts rows (map get-layout-name (cell-ref *current-layouts*))
                            current (+ number 1))
	   (xkb-mark-as-dirty)))))


(define (current-layout-name)
  (sure-string
    (list-ref (cell-ref *current-layouts*) (current-layouts current))))

(define (variant-index variant)
 	 (list-index (lambda(x)
 		       (eq? (car x) variant))
 		     (cell-ref *selected-variants*)))

(define (update-selected-variants)
  (cell-set! *selected-variants*
             (woo-catch (lambda()
                          (append (default-variants)
                                  (get-list (string-append
                                             "/xkb/layouts/"
                                             (current-layout-name)
                                             "/variants"))))
                        (lambda(reason)
                          (default-variants)))))

(define (update-variants)
  (if (>= (current-layouts current) 0)
      (begin (update-selected-variants)
             (avail-variants rows (map cdr (cell-ref *selected-variants*))
                             current (variant-index (list-ref (cell-ref *current-variants*)
                                                              (current-layouts current)))))
      (avail-variants rows-clear)))

(define (set-variants)
  (let ((number (current-layouts current))
 	(variant (avail-variants current)))
    (and (>= number 0)
         (>= variant 0)
 	 (begin
 	   (set-car! (drop (cell-ref *current-variants*) number)
 		     (car (list-ref (cell-ref *selected-variants*) variant)))
 	   (xkb-mark-as-dirty)))))

 (define (model-index model)
   (and model
        (list-index (lambda (x)
 		     (eq? (sure-symbol model) (car x)))
 		   *xkb-models*)))



;;;;;;;;;;;;;;; layout description

 (define (up-down-buttons)
   (list
     (button (_ "Up") (when clicked (up-layout)))
     (button (_ "Down") (when clicked (down-layout)))))

 (define (add-remove-buttons)
   (list
     (button (_ "Add") (when clicked (add-layout)))
     (button (_ "Remove") (when clicked (remove-layout)))))


(tabbox
   (document:id layoutbox
                (tab  (_ "Layouts")
                      (groupbox (_ "Model")
                                (document:id avail-models
                                                   (combobox
                                                    rows (map cdr *xkb-models*)
                                                    current (or (model-index (cell-ref *current-model*))
                                                                (model-index 'pc105)
                                                                0)
                                                    (when selected
                                                      (xkb-mark-as-dirty)))))
                       (groupbox (_ "Layouts")
                                 (vbox margin 5
                                       (label (_ "Available Layouts:"))
                                       (document:id avail-layouts
                                                    (listbox
                                                     rows (map get-layout-name (cell-ref *avail-layouts*))))
                                       (hbox (add-remove-buttons))
                                       (document:id current-layouts
                                                    (listbox
                                                     rows (map get-layout-name (cell-ref *current-layouts*))
                                                     (when selected (update-variants))))
                                       (hbox (up-down-buttons))))
                       (hbox
                        (label (_ "Variant:"))
                        (document:id avail-variants
                                     (combobox (when selected (set-variants)))))
                       (groupbox (or (cond-cdr (assq 'grp *xkb-option-groups*)) (_ "Switch"))
                                 (document:id  avail-switches
                                               (combobox rows (map cdr *xkb-switches*)
                                                         current (or (cell-ref *current-switch*) 0)
                                                         (when selected (xkb-mark-as-dirty)))))))
   current layoutbox)

;;;;;;;;;;;;;;;;; frame buttons

(define (get-current-options)
  (append (list (car (list-ref *xkb-switches* (avail-switches current))))
 	  (cell-ref *current-options*)))

(define (do-apply)
  (woo-catch
   (lambda()
     (woo-write "/xkb/current"
                'layout (string-join (map sure-string (cell-ref *current-layouts*)) ",")
                'variant (string-join (map sure-string (cell-ref *current-variants*)) ",")
                'model (sure-string (car (list-ref *xkb-models* (avail-models current))))
                'options (string-join (map sure-string (get-current-options)) ","))
     (woo-write "/xkb/reload")
     (xkb-clear-dirty)
     #t)
   (lambda (reason) #f)))

(frame:buttons-view
 (button (_ "Quit") (when clicked (document:end)))
 (document:id apply-button (button (_ "Apply")
                                   inactive
                                   (when clicked (do-apply)))))

